#!/bin/bash
installPath="/usr/AccessData/agent"
logfile=/tmp/agentcored.log
tmpfile=/tmp/agentcored.tmp

isSolaris="0"
if [ -e /etc/release ]; then
	isSolaris=`grep Solaris /etc/release | wc -l`
fi
isAIX=0
uname -a | grep AIX > /dev/null 2>&1
if [ $? -eq 0 ]; then
	isAIX=1
fi

INITD=/etc/init.d/agentcored


function logit()
{
	echo "$1"
	echo "$1" >> $logfile
}

showHelp()
{
	logit "agent.sh <certpath> [options]"
	logit "agent.sh -remove | -r"
	logit "agent.sh -removefully | -rf"
	logit " options: "
	logit "   -installpath,-i <installpath>"
	logit "   -lifetime,-l <lifetime>"
	logit "     if <lifetime> == 0, it will never uninstall itself"
	logit "     if <lifetime> > 0 it is in days before uninstall"
	logit "     if <lifetime> < 0 it is in minutes before uninstall"
	logit "   -port,-p <port>"
	logit "   -connections,-c <connections>"
	logit "   -size,-s <storagesize>"
	logit "   -datasize,-ds <datasize>"
	logit "   -mama,-m <ip:port>"
	logit "   -paport, -pa <port>"
	exit 1
}

uninstall()
{
	if [ $isAIX == 1 ]; then
		#TBD: Does this set INSTALLPATH?
		rmssys -s ad_agent >> $logfile 2>&1
	else
		if [ ! -e $INITD ]; then
			logit "Agent is not installed"
			exit 1
		fi
	
		#find the INSTALLPATH and stop the service
		source $INITD stop >> $logfile 2>&1

		#solaris
		if [ $isSolaris = "1" ]; then
			rm -rf /etc/rc3.d/S20agentcored > /dev/null 2>&1
			rm -rf /etc/rc2.d/K20agentcored > /dev/null 2>&1
		#ubuntu
		elif [ -e /usr/sbin/update-rc.d ]; then
			update-rc.d -f agentcored remove >> $logfile 2>&1
		#suse | redhat
		else
			/sbin/chkconfig --del agentcored >> $logfile 2>&1
		fi

		rm -rf $INITD > /dev/null 2>&1
	fi

	if [ "$1" == "full" ]; then
		rm -rf "$INSTALLPATH" > /dev/null 2>&1
	else
		rm -f "$INSTALLPATH/agentcored" > /dev/null 2>&1
		rm -f "$INSTALLPATH/libadfipscomm.so" > /dev/null 2>&1
		rm -f "$INSTALLPATH/installer.sh" > /dev/null 2>&1
	fi
		
	exit 0;
}

if [ "$#" -eq 0 ]; then
	showHelp
	exit 0;
fi

CERTPATH=
lifetime="0"
port="3999"
connections="10"
storagesize="16777216"
datasize="268435456"
mama="0"
paport="0"
while [ -n "$1" ]; do
	case $1 in
		-h|/?)
			showHelp
			;;
		-remove|-r)
			uninstall
			;;
		-removefully|-rf)
			uninstall full
			;;
		-installpath|-i)
			shift
			if [ -z "$1" ]; then logit "ERROR: missing install path"; showHelp; fi 
			installPath=$1
			;;
		-lifetime|-l)
			shift
			if [ -z "$1" ]; then logit "ERROR: missing lifetime"; showHelp; fi 
			lifetime=$1
			;;
		-port|-p)
			shift
			if [ -z "$1" ]; then logit "ERROR: missing port"; showHelp; fi 
			port=$1
			;;
		-connections|-c)
			shift
			if [ -z "$1" ]; then logit "ERROR: missing connections"; showHelp; fi 
			connections=$1
			;;
		-size|-s)
			shift
			if [ -z "$1" ]; then logit "ERROR: missing storage size"; showHelp; fi 
			storagesize=$1
			;;
		-datasize|-ds)
			shift
			if [ -z "$1" ]; then logit "ERROR: missing data size"; showHelp; fi
			datasize=$1
			;;
		-mama|-m)
			shift
			if [ -z "$1" ]; then logit "ERROR: missing mama value"; showHelp; fi
			mama=$1
			;;
		-paport|-pa)
			shift
			if [ -z "$1" ]; then logit "ERROR: missing paport value"; showHelp; fi
			paport=$1
			;;
		*)
			if [ -n "$CERTPATH" ]; then logit "ERROR: unrecognized parameter: $1"; showHelp; fi
			CERTPATH=$1
			;;
	esac
	shift
done

if [ ! -e "$CERTPATH" ]; then
	logit "certificate file $CERTPATH not found"
	exit 1;
fi

#find the abosolute file path and file name
OLDWD=`pwd`

mkdir -p "$installPath" > /dev/null 2>&1
if [ ! -d "$installPath" ]; then
	logit "Could not create install dir"
	exit 1;
fi

#extract to a tmp dir
TMPDIR=/tmp/agentcore.$$
mkdir $TMPDIR > /dev/null 2>&1
if [ ! -d $TMPDIR ]; then
	logit "Could not create tmp dir"
	exit 1;
fi
SKIP=`awk '/^__ARCHIVE_FOLLOWS__/ { print NR + 1; exit 0; }' $0`
TAIL="tail -n+$SKIP"
if [ $isSolaris = "1" ] || [ "$isAIX" == "1" ]; then
	TAIL="tail +$SKIP"
fi
pwd
$TAIL $0 | gzip -cd > $tmpfile
cd $TMPDIR; tar xf $tmpfile
rm $tmpfile > /dev/null 2>&1
chown -R root:root * > /dev/null 2>&1

cp $TMPDIR/* "$installPath" >> $logfile 2>&1
rm -fR "$installPath/tmp" > /dev/null 2>&1

#register the service
if [ $isSolaris = "1" ]; then
	mv -f $TMPDIR/tmp/agentcored.sun $INITD > /dev/null 2>&1
	cd /etc/rc3.d/; ln -s $INITD S20agentcored > /dev/null 2>&1
	cd /etc/rc2.d/; ln -s $INITD K20agentcored > /dev/null 2>&1
elif [ $isAIX -eq 1 ]; then 
	echo '#!/bin/bash' > $installPath/agentcore
	echo env LIBPATH=$installPath $installPath/agentcored -d >> $installPath/agentcore
	chmod +x $installPath/agentcore
	mkssys -s ad_agent -p $installPath/agentcore -u 0 -R -Q -G ad -d >> $logfile 2>&1
#ubuntu
elif [ -e /usr/sbin/update-rc.d ]; then
	mv -f $TMPDIR/tmp/agentcored.ubuntu $INITD > /dev/null 2>&1
	chmod +x $INITD > /dev/null 2>&1
	/usr/sbin/update-rc.d agentcored defaults >> $logfile 2>&1

#suse
elif [ -e /etc/SuSE-release ]; then
	mv -f $TMPDIR/tmp/agentcored.suse $INITD > /dev/null 2>&1
	chmod +x $INITD > /dev/null 2>&1
	/sbin/chkconfig agentcored 345 >> $logfile 2>&1

#redhat
else
	mv -f $TMPDIR/tmp/agentcored.redhat $INITD > /dev/null 2>&1
	chmod +x $INITD > /dev/null 2>&1
	/sbin/chkconfig --add agentcored > /dev/null 2>&1
fi

if [ $isAIX -ne 1 ]; then
	#stamp in the install path into the init.d file
	REGEX="s|STAMPPATH|$installPath|"
	sed "$REGEX" $INITD > $tmpfile
	mv $tmpfile $INITD > /dev/null 2>&1
	chmod +x $INITD > /dev/null 2>&1
fi

#run test_setup
chmod +x $TMPDIR/tmp/test_setup > /dev/null 2>&1
cd $OLDWD
if [ $isSolaris = "1" ]; then export LD_LIBRARY_PATH=$installPath:$LD_LIBRARY_PATH; fi
if [ $isAIX -eq 1 ]; then export LIBPATH=$installPath:$LIBPATH; fi
$TMPDIR/tmp/test_setup "$installPath" $datasize "$CERTPATH" "$TMPDIR/tmp/adata.p7b" $port $connections $lifetime $storagesize $mama $paport >> $logfile 2>&1
if [ "$?" -ne "0" ]; then
	logit "WARNING--storage setup failed"
fi

if [ $isAIX -eq 1 ]; then
	startsrc -s ad_agent >> $logfile 2>&1
else
	$INITD start >> $logfile 2>&1
fi

#clean up tmp dir
rm -rf $TMPDIR > /dev/null 2>&1
exit 0

__ARCHIVE_FOLLOWS__
